import os
import uuid
import sys
import raava

from raava.rules import get_handlers
from raava.rules import EventRoot
from raava.handlers import Loader
from gns.env import setup_config


class GNSCheckerException(Exception):
    pass


def __monkey_patch():
    """
    Patch raava worker to avoid import issue
    """
    class Mock():

        def __init__(self):
            pass

        def checkpoint(self):
            pass

        def get_current_task(self):
            return self

    raava.worker = Mock()


def _import_module(path_to_module: str):
    """
    Import modules from path
    returns set of functions named 'on_event'
    which fined in path
    """
    checked_fn_name = 'on_event'
    abspath_to_module = os.path.abspath(path_to_module)
    sys.path.append(abspath_to_module)
    loader = Loader(abspath_to_module, [checked_fn_name])
    functions_to_check = loader.get_handlers('').get(checked_fn_name)
    if not functions_to_check:
        raise GNSCheckerException("Can't find function {name}".format(name=checked_fn_name))
    else:
        return functions_to_check


def _get_event_root(event_desc: dict):
    """
    Return event object, which GNS expected
    """
    event = EventRoot()
    event.set_extra({'handler': 'on_event',
                     'job_id': str(uuid.uuid4()),
                     'counter': 0})

    event.update(event_desc)
    return event


def check(config: dict, rule_path: str, event_desc: dict):
    """
    Search for functions in GNS, which mapped for this event
    and execute it
    """
    setup_config(config)
    __monkey_patch()
    event_root = _get_event_root(event_desc)
    on_event_fn = _import_module(rule_path)

    matched_tasks = get_handlers(event_root, {'on_event': on_event_fn})
    for task in matched_tasks:
        try:
            task(event_root)
        except Exception as error:
            raise GNSCheckerException("Can't execute rule {}, by reason: {}".format(task, error))



import json
import click
import os
import sys
import logging
import pprint
from gnscli import uploader
from gnscli import gnsapi
from gnscli import checker
from gnscli import settings


LOG = logging.getLogger(__name__)


def _validate_repo_path(_, value):
    if '.git' not in os.listdir(value):
        raise click.BadParameter(
            "{repo_path} is not git repository!".format(repo_path=value))
    else:
        return value


def _validate_event_desc(_, event_file):
    try:
        event_desc = json.load(event_file)
    except (TypeError, ValueError):
        LOG.error("Can't parse event description file %s", event_file)
    else:
        return event_desc


@click.group()
@click.option('--debug/--no-debug', default=False)
@click.option('--config', '-c', type=click.File('r'), callback=settings.Settings.load)
def cli(debug, config):
    """
    GNS command line tool.
    """
    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)


@cli.group()
@click.option('--rules-path', '-r', type=click.Path(exists=True), envvar='GNS_RULES_PATH',
              callback=_validate_repo_path, default='.', help="Path to rules dir")
@click.pass_context
def rules(ctx, rules_path):
    """
    Manage GNS rules
    """
    ctx.obj = rules_path


@rules.command()
@click.option('--message', '-m', required=True, help="Describe you changes")
@click.option('--gns-server', envvar='GNS_SERVER', required=True, help="GNS FQDN")
@click.pass_obj
def upload(rules_path, gns_server, message):
    """
    Upload new or changed rules in GNS
    """
    LOG.info("Upload updated rules to GNS...")
    uploader.upload(gns_server, rules_path, message)


@rules.command()
@click.option('--event-desc', '-e', required=True, type=click.File('r'),
              callback=_validate_event_desc, help="JSON file with event description")
@click.pass_obj
def execute(rules_path, event_desc):
    """
    Run GNS rules locally
    """
    config = settings.Settings.config
    checker.check(config, rules_path, event_desc)


@cli.group()
@click.option('--gns-server', envvar='GNS_SERVER', required=True, help="GNS FQDN")
@click.pass_context
def gns(ctx, gns_server):
    """
    GNS API wrapper
    """
    ctx.obj = gns_server


@gns.command()
@click.pass_obj
def cluster_info(gns_server):
    """
    Show generic cluster info
    Equivalent to  GET `/system/state`
    """
    gns_state = gnsapi.get_cluster_info(gns_server)
    click.echo(pprint.pformat(gns_state))


@gns.command()
@click.pass_obj
def jobs_list(gns_server):
    """
    Show current jobs list by id
    Equivalent to  GET `/jobs/`
    """
    jobs = gnsapi.get_jobs(gns_server)
    click.echo(pprint.pformat(jobs))


@gns.command()
@click.argument('job_id')
@click.pass_obj
def kill_job(gns_server, job_id):
    """
    Terminate job by id.
    Now, by GNS API limitation, job just marked as `should be deleted`,
    physically it could be deleted for several time or never.
    """
    gnsapi.terminate_job(gns_server, job_id)


@gns.command()
@click.argument('host', required=False)
@click.argument('service', required=False)
@click.argument('severity', required=False)
@click.option('--file', '-f', type=click.File('r'), help="Path to JSON file with event description")
@click.pass_obj
def send_event(gns_server, host, service, severity, file):
    """
    Send event to GNS via API.
    Could be called with arguments `host service severity` or with JSON file event description.
    """

    if file:
        with file:
            event = json.load(file)
    elif host and service and severity:
        event = {'host': host, 'service': service, 'severity': severity}
    else:
        LOG.error("You mast pass `host service severity` args or --file option")
        sys.exit(1)

    LOG.info("Send event: {}".format(pprint.pformat(event)))

    gnsapi.send_event(gns_server, event)


def main():
    """
    Command's entry point
    """
    try:
        cli()
    except (gnsapi.GNSAPIException, uploader.GitCommandError) as error:
        LOG.error("Error occurred: %s", error)
        sys.exit(1)



import os
from pkg_resources import resource_stream
import yaml


class Settings:

    config = None

    @classmethod
    def load(cls, ctx, param, file):
        """This callback loads config from file, if option `--config/-c` is defined,
           in any other cases loads config by default paths"""
        if file:
            cls.config = yaml.load(file)
        else:
            config = yaml.load(resource_stream(__name__, 'config.yaml'))
            path_to_user_config = os.path.expanduser('~/.config/gnscli/config.yaml')
            if os.path.exists(path_to_user_config):
                config.update(yaml.load(open(path_to_user_config)))
            cls.config = config



"""
This module is wrapper to GNS REST API.
"""

import json
import requests
from requests.compat import urljoin
import logging


LOG = logging.getLogger(__name__)


class GNSAPIException(Exception):
    pass


def get_cluster_info(gns_server: str):
    resp = requests.get(urljoin(gns_server, '/api/rest/v1/system/state'))
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't get cluster info")
    else:
        return resp.json()


def send_event(gns_server: str, event_desc: dict):
    resp = requests.post(urljoin(gns_server, '/api/rest/v1/jobs'),
                         headers={'content-type': 'application/json'},
                         data=json.dumps(event_desc))
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't post new event.", resp.text)
    else:
        LOG.info("New event posted. Job Id: {}".format(resp.json().get('id')))


def terminate_job(gns_server: str, job_id: str):
    resp = requests.delete(urljoin(gns_server, '/api/rest/v1/jobs/{}'.format(job_id)))

    if resp.status_code == 404:
        LOG.info("Job id `{}` is not found. Probably it is already deleted or was not created".format(job_id))
        return
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't delete job id {}".format(job_id), resp.text)
    else:
        LOG.info("Job id {} was deleted".format(job_id))


def set_header(gns_server: str, head: str):
    resp = requests.post(urljoin(gns_server, '/api/rest/v1/rules/head'),
                         headers={'content-type': 'application/json'},
                         data=json.dumps({"head": head}))
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't upload new HEAD.", resp.text)
    else:
        LOG.info("Set new head: {}".format(head))


def get_header(gns_server: str):
    resp = requests.get(urljoin(gns_server, '/api/rest/v1/rules/head'))
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't upload new HEAD.")
    else:
        return resp.json().get('head')


def get_jobs(gns_server: str):
    resp = requests.get(urljoin(gns_server, '/api/rest/v1/jobs'))
    try:
        resp.raise_for_status()
    except requests.HTTPError:
        raise GNSAPIException("Can't get jobs list")
    else:
        return resp.json()



"""
This module is for upload updated or new rules to GNS.
"""

import envoy
from gnscli import gnsapi
import logging
from gnscli.settings import Settings

logger = logging.getLogger(__name__)


class GitCommandError(Exception):
    pass


def _update_head(gns_server: str, path: str):
    new_head = _execute_git_command("rev-parse HEAD", path, err_msg="Can't update HEAD").strip()
    current_head = gnsapi.get_header(gns_server)
    if new_head == current_head:
        logger.info("HEAD already updated")
        return
    gnsapi.set_header(gns_server, new_head)


def _execute_git_command(cmd: str, path, err_msg: str):
    git_warn_exit_code = 1
    full_cmd = 'git --git-dir={path}/.git --work-tree={path} {cmd}'.format(cmd=cmd, path=path)
    logger.debug("Execute command: %s", full_cmd)
    result = envoy.run(full_cmd)
    out, err, exit_code = result.std_out, result.std_err, result.status_code
    logger.debug("Git stdout: %s", out)
    if exit_code > git_warn_exit_code:
        raise GitCommandError(err_msg, err, out, full_cmd)
    elif exit_code == git_warn_exit_code:
        logger.warning("Git stderr: %s", err)
        return out
    else:
        return out


def upload(gns_server: str, path: str, message: str):
    """
    This function execute git commands:
        - git commit -a -m "{message}"
        - git pull --rebase
        - git push # to origin
        - git push ssh://git@gns/remote.git # to each GNS git
    and POST new HEAD hash to GNS via API
    """
    status = _execute_git_command('status', path, "Can't get git status").split('\n')

    if 'nothing to commit, working directory clean' in status[3]:
        logger.info("There is no changes.")

    logger.info("Commit current changes...")

    _execute_git_command('commit -a -m "{}"'.format(message), path,
                         "Can't commit your changes")

    logger.info("Pull changes from rules server...")

    _execute_git_command('pull --rebase', path, "Can't pull changes from server rules")

    logger.info("Sync you changes with rules server...")

    _execute_git_command('push', path, "Can't push your changes")

    gns_repos = Settings.config.get("gns_git_remotes")
    for repo in gns_repos:
        logger.info("Upload rules to {}...".format(repo))
        _execute_git_command('push {} master'.format(repo), path, err_msg="Can't push to GNS remote")

    _update_head(gns_server, path)

    logger.info("You rules uploaded to GNS!")



#!/usr/bin/env python

from setuptools import setup

setup(name='gnscli',
      version='0.6.1',
      description='GNS Command line tool',
      author='Alexander Kushnarev',
      author_email='avkushnarev@gmail.com',
      url='https://github.com/yandex-sysmon/gns-rules-checker',
      packages=['gnscli'],
      package_data={'gnscli': ['config.yaml']},
      entry_points={'console_scripts': ['gns-cli = gnscli.client:main']},
      install_requires=['gns', 'raava', 'pyyaml', 'click', 'envoy-beta', 'requests'])
